/*
 *= Rohde&Schwarz Digital Radiocommunication Tester Program Example ===========
 *
 * C Example Program 2: GSM-MS Signalling Measurements
 *
 * The following example explains how to handle primary and secondary
 * addressing and how to perform a simple measurement with the CMU. In the
 * example, remote control via GPIB bus and the programming language C is used.
 *
 * The following measurement example require option CMU-K21-K22 Version >= 2.02
 *
 * A GSM mobile phone is connected to the bidirectional connector RF2 and
 * tested. We assume that the mobile phone is capable of operating in the
 * GSM900/1800 band, an appropriate dual-band mobile must be used. Moreover, a
 * test SIM card for the mobile phone is required.
 *
 * Compiler name and release(s) : LabWindows/CVI 5.0
 * Original Release : January, 2000
 *
 * By ROHDE & SCHWARZ FRANCE:
 *      CONTACTS :
 *              Support & Application
 *              Rohde & Schwarz France
 *              Parc Tertiaire de Meudon
 *              25-27, rue Jeanne Braconnier
 *              92366 Meudon-la-Foret
 *              Telephone : 00 33 1 41 36 10 29
 *              Fax       : 00 33 1 41 36 11 10
 *              e-Mail    : GSA@RSF.RSD.DE
 *
 * Version : 1.2
 *
 * Copyright 2000 ROHDE & SCHWARZ FRANCE. All Rights Reserved.
 *
 * ----------------------------------------------------------------------------
 *                         MODIFICATON HISTORY
 * ----------------------------------------------------------------------------
 *  Date      | Author      | Description
 *    Release |     Company |            
 * ----------------------------------------------------------------------------
 *  26.Jan.00 | S.Blanchon  | Creation (accordingly to the Operating Manual
 *       V1.0 |         RSF | Edition 04).
 *            |             | 
 *            |             | Delivery version 1.0            
 * ----------------------------------------------------------------------------
 *  31.Mar.00 | S.Blanchon  | Modified accordingly to the CMU LabWindows/CVI
 *       V1.1 |         RSF | Instrument Driver Version 2.02.
 *            |             | 
 *            |             | Delivery version 1.1            
 * ----------------------------------------------------------------------------
 *  30.May.00 | S.Blanchon  | Modified accordingly to the splitted CMU
 *       V1.2 |         RSF |  LabWindows/CVI Instrument Driver Version 2.03.
 *            |             | 
 *            |             | Delivery version 1.2
 *=============================================================================
 */
#include <utility.h>
#include <ansi_c.h>
#include <string.h>
#include <formatio.h>
#include <visa.h>
#include <userint.h>
#include <cvirte.h> /* Needed if linking in external compiler; harmless otherwise */

// Include R&S CMU driver header file
#include "RSCMU200.h"
#include "RSCMUK2G.h"

// Specific timeout measurement definition
#define SPOW_TIMEOUT    2.0
#define POWMPR_TIMEOUT  5.0
#define BER_TIMEOUT    10.0

// Variable declaration for secondary address for Base definition and for
// Function Groups definition
static ViSession    g_hRsCmuBase  = VI_NULL;        // Basic Unit
static ViSession    g_hRsCmuRf_NSig = VI_NULL;      // RF Non-Signalling Unit
static ViSession    g_hRsCmuGsm900_Sig = VI_NULL;   // GSM  900 Signalling Unit 
static ViSession    g_hRsCmuGsm900_NSig = VI_NULL;  // GSM  900 Non-Signalling Unit
static ViSession    g_hRsCmuGsm1800_Sig = VI_NULL;  // GSM 1800 Signalling Unit
static ViSession    g_hRsCmuGsm1800_NSig = VI_NULL; // GSM 1800 Non-Signalling Unit

// Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif
// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
    ViChar l_vCmuErrMsg[256];
    ViChar l_vErrBuf[512];
    ViInt32 _error;
    
    if (CmuStatus != VI_SUCCESS)
    {
        RSCMU_K2G_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
        if (CmuStatus == VI_ERROR_TMO) RSCMU_ErrorQuery(g_hRsCmuBase, &CmuStatus, l_vCmuErrMsg);
        Fmt (l_vErrBuf, "%s<Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);
        MessagePopup ("ERROR MESSAGE (CMU)", l_vErrBuf);
        return -1;
    }
    return 0;
    
Error:
    return -1;
}

// Macro "WriteCmuViInt32Value" definition (Check if no result could be
// obtained (yet), or if the result obtained is invalid).
#ifndef WriteCmuViInt32Value
#define WriteCmuViInt32Value(String,ViInt32Value) \
(ViInt32Value != RSCMU_NAN_VI_INT32) ? printf ("%s%d\n", String, ViInt32Value) : printf ("%sNot a number\n", String);
#endif

// Macro "WriteCmuViReal64Value" definition (Check if no result could be
// obtained (yet), or if the result obtained is invalid).
#ifndef WriteCmuViReal64Value
#define WriteCmuViReal64Value(String,ViReal64Value) \
(ViReal64Value != RSCMU_NAN) ? printf ("%s%lf\n", String, ViReal64Value) : printf ("%sNot a number\n", String);
#endif

/*****************************************************************************/
/*======================== Main =============================================*/
/*****************************************************************************/
int main (int argc, char *argv[])
{                                  
  ViInt32    _error;
  
  clock_t    StartTime;
  clock_t    CurrentTime;
  
  ViChar     l_vInstrDriverRev[256];
  ViChar     l_vInstrFirmwareRev[256];
  
  ViChar     errorMessage[256];
  
  ViInt32    l_vSignallingState;
  
  ViInt32    l_vSpowStatus = RSCMU_MEAS_OFF;
  ViInt32    l_vSensStatus = RSCMU_MEAS_OFF;
  ViInt32    l_vPowMprStatus = RSCMU_MEAS_OFF;
  ViInt32    l_vCurrentStatCycleCount;
  ViInt32    l_vCurrentEvalPeriodCount;
  
  ViInt32    l_vSensMode = RSCMU_RFER; // or RSCMU_BBB for Burst-By-Burst BER (Fast BER)
  
  ViReal64   l_vPkPwr;
  
  ViReal64   l_vAvgBurstPwCurr;
  ViReal64   l_vPeakBurstPwCurr;
  ViInt32    l_vPowerControlLevel;
  ViReal64   l_vTimingAdvError;
  ViReal64   l_vBurstsOutOfPowTol;
  ViInt32    l_vBurstMatching;
  ViReal64   l_vAvgBurstPwAvg;
  
  ViReal64   l_vPhErrPeakMaxMin;
  ViReal64   l_vPhErrRMSMaxMin;
  ViReal64   l_vFreqErrMaxMin;
  ViReal64   l_vBurstsOutOfModTol;
  
  ViInt32    l_vRxLev;
  ViInt32    l_vRxQual;
  
  ViReal64   l_vProgressTime;
  ViReal64   l_vClassIIBits;
  ViReal64   l_vClassIbBits;
  ViReal64   l_vFer;
  ViInt32    l_vCRCErrors;
  ViInt32    l_vStatus;
  
  if (InitCVIRTE (0, argv, 0) == 0) /* Needed if linking in external compiler; harmless otherwise */
    return -1; /* out of memory */

  Cls();
  
  
  // ==========================================================================
  // CMU BASIC UNIT
  // --------------------------------------------------------------------------
  // BASIC UNIT INITIALISATION
  //  -> GPIB board index is assigned to 0
  //  -> Primary address to the GPIB bus is assigned to the default (20)
  //  -> Ask for the indentifier of the CMU, reset the instrument
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Init_Base (&g_hRsCmuBase, RSCMU_GPIB, 0, 20, RSCMU_DO_QUERY, RSCMU_RESET_ON, errorMessage));
  
  // --------------------------------------------------------------------------
  // FUNCTION GROUPS CONFIGURATION
  //   Subsystem Remote
  //     -> Secondary addresses of the GPIB bus are assigned for all available
  //        function groups
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 1, RSCMU_RF_NSIG, errorMessage));
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 2, RSCMU_GSM900MS_SIG, errorMessage));
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 3, RSCMU_GSM900MS_NSIG, errorMessage));
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 4, RSCMU_GSM1800MS_SIG, errorMessage));
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 5, RSCMU_GSM1800MS_NSIG, errorMessage));
  
  // --------------------------------------------------------------------------
  // FUNCTION GROUPS INITIALISATION
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Init_RF_NSig (g_hRsCmuBase, &g_hRsCmuRf_NSig, errorMessage));
  CheckCmuError (RSCMU_Init_GSM_Sig (g_hRsCmuBase, &g_hRsCmuGsm900_Sig, RSCMU_GSM900, errorMessage));
  CheckCmuError (RSCMU_Init_GSM_NSig (g_hRsCmuBase, &g_hRsCmuGsm900_NSig, RSCMU_GSM900, errorMessage));
  CheckCmuError (RSCMU_Init_GSM_Sig (g_hRsCmuBase, &g_hRsCmuGsm1800_Sig, RSCMU_GSM1800, errorMessage));
  CheckCmuError (RSCMU_Init_GSM_NSig (g_hRsCmuBase, &g_hRsCmuGsm1800_NSig, RSCMU_GSM1800, errorMessage));
  
  // --------------------------------------------------------------------------
  //     -> Set a specified timeout value for all available function groups.
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuGsm900_Sig, 10000, errorMessage));
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuGsm900_NSig, 10000, errorMessage));
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuGsm1800_Sig, 10000, errorMessage));
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuGsm1800_NSig, 10000, errorMessage));
  
  // --------------------------------------------------------------------------
  
  CheckCmuError (RSCMU_Base_RevisionQuery (g_hRsCmuBase, l_vInstrDriverRev, l_vInstrFirmwareRev, errorMessage));
  printf ("Instrument Driver Revision (Base)       : %s\n", l_vInstrDriverRev);
  
  CheckCmuError (RSCMU_K2G_RevisionQuery (g_hRsCmuBase, l_vInstrDriverRev, l_vInstrFirmwareRev, errorMessage));
  printf ("Instrument Driver Revision (K2G Option) : %s\n", l_vInstrDriverRev);
  
  printf ("Instrument Firmware Revision            : %s\n", l_vInstrFirmwareRev);
  printf ("\n");
  
  // Opens the file "RsCmu.iee" for output IEEE bus commands.
  CheckCmuError (RSCMU_Trace_ON (g_hRsCmuBase, "RsCmu.iee", errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // Example for a Dual-Band mobile in the GSM900 and GSM1800 band
  // ..........................................................................
  // Preliminary configurations for a power and sensitivity measurement are
  // defined, and the network parameters are set for maximum speed of the call
  // procedure. The IMSI of the mobile phone must be known to set up a call;
  // it can be either reported to the tester or will be determined during the
  // location update. The first alternative will speed up the call procedure.
  // Next, the CMU generates a BCCH (C0 carrier) signal for synchronization.
  // The mobile phone searches the whole channel range for this BCCH and camps
  // on it for some mobile-specific time until it reaches the Synchronized
  // state. In this state, configuration settings made at the beginning are
  // checked, and the CMU sets up a call to the mobile phone. Parameters such
  // as the PCL and channel should be defined before the call is setup to
  // reduce signalling time (no PCL/channel change). As soon as the call is
  // established, power and sensitivity measurements are performed.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  // --------------------------------------------------------------------------
  //   Synchronisation
  //     -> Frequency of the synchronization signal (10MHz)
  //     -> Internal reference frequency used
  // --------------------------------------------------------------------------    
  CheckCmuError (RSCMU_Base_ConfSyncFreqRef (g_hRsCmuBase, 10.0, RSCMU_INTERNAL, errorMessage));
  
  // ==========================================================================
  // CMU K22 UNIT (GSM1800)
  // --------------------------------------------------------------------------
  //         Signalling - Group Configuration
  //               Subsystem LEVel (Input Level)
  //               -> Set the input level according to the power control level of
  //                  the mobile. 
  //               -> Set the low noise attenuation for the RF input signal (full
  //                  dynamic range)
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_SensLevMode (g_hRsCmuGsm1800_Sig, RSCMU_PCL, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_SensLevAtt (g_hRsCmuGsm1800_Sig, RSCMU_LOW_NOISE, errorMessage));
  
  // --------------------------------------------------------------------------
  //         Signalling - Connection Control
  //               Subsystem SIGNalling (Call Setup and Cleardown)
  //               -> Switch off BCCH signal
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ProcSignAct (g_hRsCmuGsm1800_Sig, RSCMU_SOFF, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem NETWork:MS (Mobile Settings)
  //               -> Permits the mobile to switch off the DTX mode (discontinuous mode)
  //               -> Definition of the power level upon registration in the network:
  //                     PCL 0 for GSM1800 network 
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwDtx (g_hRsCmuGsm1800_Sig, RSCMU_OFF, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwPow (g_hRsCmuGsm1800_Sig, 0, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem NETWork:REQuest (Request Mobile Data)
  //               Note: selected for maximum speed of the call procedure.
  //               -> International mobile subscriber identity of the MS is requested
  //               -> International mobile station equipment identity of the MS is not requested
  //               -> Authentication request of the MS is not made
  //               -> Capability to perform a handover is not requested on the 
  //                  connected MS
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwReqImsi (g_hRsCmuGsm1800_Sig, RSCMU_OFF, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwReqImei (g_hRsCmuGsm1800_Sig, RSCMU_OFF, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwReqAuth (g_hRsCmuGsm1800_Sig, RSCMU_OFF, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwReqHand (g_hRsCmuGsm1800_Sig, RSCMU_OFF, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem NETWork:IDENtity
  //               -> Network Color Code
  //               -> Mobile Color Code
  //               -> Mobile Network Code
  //               -> Base station Color Code
  //               -> Location Area Code
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwIden (g_hRsCmuGsm1800_Sig, 0, 1, 1, 0, 1, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem IOConnector (External Attenuation at the connectors)
  //               -> Connector RF2 used as input.
  //               -> External Attenuation 0.2 dB at the Connector RF2 to
  //                  compensate for known cable losses.
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_Inp (g_hRsCmuGsm1800_Sig, RSCMU_INPUT_RF2, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_Outp (g_hRsCmuGsm1800_Sig, RSCMU_OUTPUT_RF2, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_SensCorrLossInp (g_hRsCmuGsm1800_Sig, RSCMU_INPUT_RF2, 5.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_SourCorrLossOutp (g_hRsCmuGsm1800_Sig, RSCMU_OUTPUT_RF2, 5.0, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem NETWork:SMODe (Type of Signalling)
  //               -> Definition of the international mobile subsrciber identity 
  //                  (IMSI) which serves as the default setting for a call to
  //                  mobile.
  //                  Warning: the IMSI may differ !
  //               -> Fast power change. The power change is controlled via 
  //                  FACCH (fast associated conrol channel).
  //               -> Signalling via fast associated control channel.
  //               -> Standard full rate coding (Full Rate Version 1) for the speech
  //                  and voice transmission in the traffic channels.
  //               -> Loop is closed only for BER measurement.
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodImsi (g_hRsCmuGsm1800_Sig, 1, 1, "0000000001", errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodPch (g_hRsCmuGsm1800_Sig, RSCMU_POWERCHANGE_FAST, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodSch (g_hRsCmuGsm1800_Sig, RSCMU_SIGN_FACCH, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodTraf (g_hRsCmuGsm1800_Sig, RSCMU_SPEECH_CODING_FRV1, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodLcom (g_hRsCmuGsm1800_Sig, RSCMU_LOOP_BER, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem BSSignal (Signal of Base Station/CMU)
  //               -> Control channel for the BCCH 
  //                    GSM1800: Channel 740
  //               -> Level in dBm of the control channel for the BCCH (-85.0dBm)
  //               -> Used timselot for the traffic channel (TS 3)
  //               -> Absolute level in the used timeslot (-85.0dBm)
  //               -> Reference level in the unused timeslot (-20.0dB)
  //               -> Traffic channel number
  //                    GSM1800: Traffic Channel 512
  //               -> Type of data transmitted in the traffic channel (pseudo random
  //                  sequence 9)
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfBssCchChan (g_hRsCmuGsm1800_Sig, 740, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssCchLevAbs (g_hRsCmuGsm1800_Sig, -85.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssTchTim (g_hRsCmuGsm1800_Sig, 3, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssTchLevUtim (g_hRsCmuGsm1800_Sig, -85.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssTchLevUnt (g_hRsCmuGsm1800_Sig, -20.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssTchChan (g_hRsCmuGsm1800_Sig, 512, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssSpe (g_hRsCmuGsm1800_Sig, RSCMU_SPEECH_PR9, errorMessage));
  
  // --------------------------------------------------------------------------
  //         Signalling - Test Object MODulation 
  //              Test Configuration - Subsystem MODulation:CONTrol
  //              -> Scalar values
  //                 20 bursts per statistics cycle
  //              -> Single shot measurement (until status = RDY)
  //                 Continue measurement even if case of error
  //                 Continue measurement according to its repetition mode
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfModCont (g_hRsCmuGsm1800_Sig, RSCMU_SCALAR, 20, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_ConfModContRep (g_hRsCmuGsm1800_Sig, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, errorMessage));
  
  // --------------------------------------------------------------------------
  //         Signalling - Test Object SENSitivity
  //              Test Configuration - Subsystem SENSitivity:CONTrol
  //              -> RFER Bit Error Rate (10000 bits class II)
  //                 128 bursts is sent in a single shot mode
  //                 or
  //                 Burst-By-Burst Bit Error Rate, 
  //                 88 bursts is sent in a single shot mode
  //              -> Measurement over all frames
  //                 Continue measurement according its repetition mode
  //              -> Absolute level of the TCH in the used timeslot
  //                   GSM1800: -100.0 dBm
  //              -> Relative level of the TCH in the unused timeslot
  //              -> Holdoff time:
  //                 Automatic gain control: time during which the mobile can set to
  //                 the new RF level at the beginning of the BER measurement.
  //                 Bit stream synchronization: time needed to send back the bits
  //                 received.
  // --------------------------------------------------------------------------
  switch (l_vSensMode)
  {
    case RSCMU_RFER :
      CheckCmuError (RSCMU_GSM_Sig_ConfSensCont (g_hRsCmuGsm1800_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, RSCMU_RFER, 128, errorMessage));
    break;
    case RSCMU_BBB :
      CheckCmuError (RSCMU_GSM_Sig_ConfSensCont (g_hRsCmuGsm1800_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, RSCMU_BBB, 88, errorMessage));
    break;
  }
  CheckCmuError (RSCMU_GSM_Sig_ConfSensContRep (g_hRsCmuGsm1800_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfSensContLevUtim (g_hRsCmuGsm1800_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, -100.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfSensContLevUnt (g_hRsCmuGsm1800_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, -20.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfSensContHtim (g_hRsCmuGsm1800_Sig, 0.5, 0.2, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfSensTset (g_hRsCmuGsm1800_Sig, RSCMU_TS1, errorMessage));
  
  // ==========================================================================
  //       CMU K21 UNIT (GSM900)
  // --------------------------------------------------------------------------
  //         Signalling - Group Configuration
  //               Subsystem LEVel (Input Level)
  //               -> Set the input level according to the power control level of
  //                  the mobile. 
  //               -> Set the low noise attenuation for the RF input signal (full
  //                  dynamic range)
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_SensLevMode (g_hRsCmuGsm900_Sig, RSCMU_PCL, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_SensLevAtt (g_hRsCmuGsm900_Sig, RSCMU_LOW_NOISE, errorMessage));

  // --------------------------------------------------------------------------
  //         Signalling - Connection Control
  //               Subsystem SIGNalling (Call Setup and Cleardown)
  //               -> Switch off BCCH signal
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ProcSignAct (g_hRsCmuGsm900_Sig, RSCMU_SOFF, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem NETWork:MS (Mobile Settings)
  //               -> Permits the mobile to switch off the DTX mode (discontinuous mode)
  //               -> Definition of the power level upon registration in the network:
  //                     PCL 5 for GSM900 network 
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwDtx (g_hRsCmuGsm900_Sig, RSCMU_OFF, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwPow (g_hRsCmuGsm900_Sig, 5, errorMessage));

  // --------------------------------------------------------------------------
  //               Subsystem NETWork:REQuest (Request Mobile Data)
  //               Note: selected for maximum speed of the call procedure.
  //               -> International mobile subscriber identity of the MS is (not) requested
  //               -> International mobile station equipment identity of the MS is (not) requested
  //               -> Authentication request of the MS is (not) made
  //               -> Capability to perform a handover is (not) requested on the 
  //                  connected MS
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwReqImsi (g_hRsCmuGsm900_Sig, RSCMU_ON, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwReqImei (g_hRsCmuGsm900_Sig, RSCMU_ON, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwReqAuth (g_hRsCmuGsm900_Sig, RSCMU_ON, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwReqHand (g_hRsCmuGsm900_Sig, RSCMU_OFF, errorMessage));

  // --------------------------------------------------------------------------
  //               Subsystem NETWork:IDENtity
  //               -> Network Color Code
  //               -> Mobile Color Code
  //               -> Mobile Network Code
  //               -> Location Area Code
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwIden (g_hRsCmuGsm900_Sig, 0, 1, 1, 0, 1, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem IOConnector (External Attenuation at the connectors)
  //               -> Connector RF2 used as input.
  //               -> External Attenuation 10.0 dB at the Connector RF2 to
  //                  compensate for known cable losses.
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_Inp (g_hRsCmuGsm900_Sig, RSCMU_INPUT_RF2, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_Outp (g_hRsCmuGsm900_Sig, RSCMU_OUTPUT_RF2, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_SensCorrLossInp (g_hRsCmuGsm900_Sig, RSCMU_INPUT_RF2, 5.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_SourCorrLossOutp (g_hRsCmuGsm900_Sig, RSCMU_OUTPUT_RF2, 5.0, errorMessage));
  
  // --------------------------------------------------------------------------
  //               Subsystem NETWork:SMODe (Type of Signalling)
  //               -> Definition of the international mobile subsrciber identity 
  //                  (IMSI) which serves as the default setting for a call to
  //                  mobile.
  //                  Warning: the IMSI may differ !
  //               -> Fast power change. The power change is controlled via 
  //                  FACCH (fast associated conrol channel).
  //               -> Signalling via fast associated conrol channel.
  //               -> Standard full rate coding (Full Rate Version 1) for the speech
  //                  and voice transmission in the traffic channels.
  //               -> Loop is closed only for BER measurement.
  //               -> ALWAYS : The mobile station always performs the location
  //                           update.
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodImsi (g_hRsCmuGsm900_Sig, 1, 1, "0000000001", errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodPch (g_hRsCmuGsm900_Sig, RSCMU_POWERCHANGE_FAST, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodSch (g_hRsCmuGsm900_Sig, RSCMU_SIGN_FACCH, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodTraf (g_hRsCmuGsm900_Sig, RSCMU_SPEECH_CODING_FRV1, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodLcom (g_hRsCmuGsm900_Sig, RSCMU_LOOP_BER, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfNetwSmodLoc (g_hRsCmuGsm900_Sig, RSCMU_ALWAYS, errorMessage));

  // --------------------------------------------------------------------------
  //               Subsystem BSSignal (Signal of Base Station/CMU)
  //               -> Control channel for the BCCH 
  //                    GSM900: Channel 31
  //               -> Level in dBm of the control channel for the BCCH (-85.0dBm)
  //               -> Used timselot for the traffic channel (TS 3)
  //               -> Absolute level in the used timeslot (-85.0dBm)
  //               -> Reference level in the unused timeslot (-20.0dB)
  //               -> Traffic channel number
  //                    GSM900: Traffic Channel 1
  //               -> Type of data transmitted in the traffic channel (pseudo random
  //                  sequence 9)
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfBssCchChan (g_hRsCmuGsm900_Sig, 31, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssCchLevAbs (g_hRsCmuGsm900_Sig, -85.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssTchTim (g_hRsCmuGsm900_Sig, 3, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssTchLevUtim (g_hRsCmuGsm900_Sig, -85.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssTchLevUnt (g_hRsCmuGsm900_Sig, -20.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssTchChan (g_hRsCmuGsm900_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfBssSpe (g_hRsCmuGsm900_Sig, RSCMU_SPEECH_PR9, errorMessage));

  // --------------------------------------------------------------------------
  //         Signalling - Test Object POWer
  //              Test Configuration - Subsystem POWer:MPR:CONTrol
  //              -> Scalar values
  //                 10 bursts per statistics cycle
  //              -> Single shot measurement (until status = RDY)
  //                 Continue measurement even if case of error
  //                 Continue measurement according to its repetition mode
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_GSM_Sig_ConfPowMprCont (g_hRsCmuGsm900_Sig, RSCMU_SCALAR, 10, errorMessage));
  CheckCmuError (RSCMU_GSM_NSig_ConfPowMprContRep (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, errorMessage));

  // --------------------------------------------------------------------------
  //         Signalling - Test Object SENSitivity
  //              Test Configuration - Subsystem SENSitivity:CONTrol
  //              -> RFER Bit Error Rate (10000 bits class II)
  //                 128 bursts is sent in a single shot mode
  //                 or
  //                 Burst-By-Burst Bit Error Rate, 
  //                 88 bursts is sent in a single shot mode
  //              -> Measurement over all frames
  //                 Continue measurement according its repetition mode
  //              -> Absolute level of the TCH in the used timeslot
  //                   GSM900: -102.0 dBm
  //              -> Relative level of the TCH in the unused timeslot
  //              -> Holdoff time:
  //                 Automatic gain control: time during which the mobile can set to
  //                 the new RF level at the beginning of the BER measurement.
  //                 Bit stream synchronization: time needed to send back the bits
  //                 received.
  // --------------------------------------------------------------------------
  switch (l_vSensMode)
  {
    case RSCMU_RFER :
      CheckCmuError (RSCMU_GSM_Sig_ConfSensCont (g_hRsCmuGsm900_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, RSCMU_RFER, 128, errorMessage));
    break;
    case RSCMU_BBB :
      CheckCmuError (RSCMU_GSM_Sig_ConfSensCont (g_hRsCmuGsm900_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, RSCMU_BBB, 88, errorMessage));
    break;
  }
  CheckCmuError (RSCMU_GSM_Sig_ConfSensContRep (g_hRsCmuGsm900_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfSensContLevUtim (g_hRsCmuGsm900_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, -102.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfSensContLevUnt (g_hRsCmuGsm900_Sig, RSCMU_TS1, RSCMU_SINGLE_SHOT, -20.0, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfSensContHtim (g_hRsCmuGsm900_Sig, 0.5, 0.2, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_ConfSensTset (g_hRsCmuGsm900_Sig, RSCMU_TS1, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // Switch ON BCCH signal 
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_Sig_ProcSignAct (g_hRsCmuGsm900_Sig, RSCMU_SON, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // Wait for location update (wait until signalling state Synchronized is
  // reached, then go to next function)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  MessagePopup ("Message", "Switch ON the Mobile Phone & Press OK to continue");
  CheckCmuError (RSCMU_GSM_Sig_WaitForMsSync (g_hRsCmuGsm900_Sig, 30, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // Call to mobile (check IMSI if it fails)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_Sig_CallToMobile (g_hRsCmuGsm900_Sig, errorMessage));
  
  printf ("GSM  900 Band\n");
  printf ("-------------\n");

  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // Channel 1, PCL 5
  //
  // Start the single shot sensitivity and the wide band peak power measurement.
  // The TX measurements are fatser and will finished first. While the
  // sensitivity is running the RX Level and RX Quality can be queried.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  printf ("\tTCH   1, PCL 5\n");
  printf ("\t--------------\n");
  
  // Initiate single shot sensitivity measurement:
  CheckCmuError (RSCMU_GSM_Sig_InitSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
  
  // Initiate the wide band peak power measurement:
  CheckCmuError (RSCMU_GSM_Sig_InitSpow (g_hRsCmuGsm900_Sig, errorMessage));

  StartTime = clock() / CLOCKS_PER_SEC;
  
  // Wait until the wide band peak power measurement is in the RDY (ready) state:
  do 
  {
    CheckCmuError (RSCMU_GSM_Sig_FetcSpowStat (g_hRsCmuGsm900_Sig, &l_vSpowStatus, &l_vCurrentStatCycleCount, errorMessage));
    switch (l_vSpowStatus)
    {
      case RSCMU_MEAS_RUN:
        break;
      case RSCMU_MEAS_RDY:
        break;
      default:
        // Abort the sensitivity and the wide band peak power measurement:
        CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
        CheckCmuError (RSCMU_GSM_Sig_AborSpow (g_hRsCmuGsm900_Sig, errorMessage));
        CheckCmuError (RSCMU_ERROR_INVALID_MEAS_STATUS);
    }
    
    CurrentTime = clock() / CLOCKS_PER_SEC;
    
    if ((CurrentTime - StartTime) > SPOW_TIMEOUT)
    {
      // Abort the sensitivity and the wide band peak power measurement:
      CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
      CheckCmuError (RSCMU_GSM_Sig_AborSpow (g_hRsCmuGsm900_Sig, errorMessage));
      CheckCmuError (RSCMU_ERROR_TIMEOUT);
    }
  } while (l_vSpowStatus != RSCMU_MEAS_RDY);
  
  // Get the wide band peak power results:
  CheckCmuError (RSCMU_GSM_Sig_FetcSpow (g_hRsCmuGsm900_Sig, &l_vPkPwr, errorMessage));
  WriteCmuViReal64Value ("\tPeak Burst Power       [dBm] = ", l_vPkPwr);
  
  // Abort the wide band peak power measurement:
  CheckCmuError (RSCMU_GSM_Sig_AborSpow (g_hRsCmuGsm900_Sig, errorMessage));
  
  StartTime = clock() / CLOCKS_PER_SEC;
  
  // Wait until the sensitivity measurement is in the RDY (ready) state:
  do
  {
    CheckCmuError (RSCMU_GSM_Sig_FetcSensStat (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, &l_vSensStatus, &l_vCurrentStatCycleCount, &l_vCurrentEvalPeriodCount, errorMessage));
    switch (l_vSensStatus)
    {
      case RSCMU_MEAS_RUN:
        break;
      case RSCMU_MEAS_RDY:
        break;
      default:
        // Abort the sensitivity measurement:
        CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
        CheckCmuError (RSCMU_ERROR_INVALID_MEAS_STATUS);
    }
    
    CurrentTime = clock() / CLOCKS_PER_SEC;
    
    if ((CurrentTime - StartTime) > BER_TIMEOUT)
    {
      // Abort the sensitivity measurement:
      CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
      CheckCmuError (RSCMU_ERROR_TIMEOUT);
    }
    
    if (l_vSensMode == RSCMU_RFER)
    {
      // Get Rx Level and Rx Quality reported by the mobile:
      CheckCmuError (RSCMU_GSM_Sig_SensRrepRxl (g_hRsCmuGsm900_Sig, &l_vRxLev, errorMessage));
      CheckCmuError (RSCMU_GSM_Sig_SensRrepRxq (g_hRsCmuGsm900_Sig, &l_vRxQual, errorMessage));
    }
  } while (l_vSensStatus != RSCMU_MEAS_RDY);
  
  if (l_vSensMode == RSCMU_RFER)
  {
    WriteCmuViInt32Value ("\tRX Level (-102.0 dBm)        = ", l_vRxLev);
    WriteCmuViInt32Value ("\tRX Quality                   = ", l_vRxQual);
  }
  
  // Get the results of the bit error rate test.
  switch (l_vSensMode)
  {
    case RSCMU_RFER :
      CheckCmuError (RSCMU_GSM_Sig_Rfer_FetcSens (g_hRsCmuGsm900_Sig,
                                                  RSCMU_SINGLE_SHOT,
                                                  &l_vProgressTime,
                                                  &l_vClassIIBits,
                                                  &l_vClassIbBits,
                                                  &l_vFer,
                                                  &l_vCRCErrors,
                                                  &l_vStatus,
                                                  errorMessage));
      WriteCmuViReal64Value ("\tClass II bits            [%] = ", l_vClassIIBits);
      WriteCmuViReal64Value ("\tFER                      [%] = ", l_vFer);
    break;
    case RSCMU_BBB :
      CheckCmuError (RSCMU_GSM_Sig_Bbb_FetcSens (g_hRsCmuGsm900_Sig,
                                                 RSCMU_SINGLE_SHOT,
                                                 &l_vProgressTime,
                                                 &l_vClassIIBits,
                                                 &l_vCRCErrors,
                                                 &l_vStatus,
                                                 errorMessage));
      WriteCmuViReal64Value ("\tClass II bits            [%] = ", l_vClassIIBits);
    break;
  }
  
  // Abort the sensitivity measurement:
  CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //      Channel 124, PCL 5
  //
  // Start the single shot sensitivity and the combined power/modulation
  // measurement. The TX measurements are fatser and will finished first.
  // While the sensitivity is running the RX Level and RX Quality can be
  // queried.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  printf ("\tTCH 124, PCL 5\n");
  printf ("\t--------------\n");
  
  // Change the traffic channel without dropping the call
  CheckCmuError (RSCMU_GSM_Sig_ProcSignTchChan (g_hRsCmuGsm900_Sig, 62, errorMessage));
  
  // Initiate single shot sensitivity measurement: 
  CheckCmuError (RSCMU_GSM_Sig_InitSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
  
  // Initiate the combined power/modulation measurement:
  CheckCmuError (RSCMU_GSM_Sig_InitPowMpr (g_hRsCmuGsm900_Sig, errorMessage));

  StartTime = clock() / CLOCKS_PER_SEC;
  
  // Wait until the combined power/modulation measurement is in the RDY (ready) state:
  do 
  {
    CheckCmuError (RSCMU_GSM_Sig_FetcPowMprStat (g_hRsCmuGsm900_Sig, &l_vPowMprStatus, &l_vCurrentStatCycleCount, &l_vCurrentEvalPeriodCount, errorMessage));
    switch (l_vPowMprStatus)
    {
      case RSCMU_MEAS_RUN:
        break;
      case RSCMU_MEAS_RDY:
        break;
      default:
        // Abort the sensitivity and the combined power/modulation measurement:
        CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
        CheckCmuError (RSCMU_GSM_Sig_AborPowMpr (g_hRsCmuGsm900_Sig, errorMessage));
        CheckCmuError (RSCMU_ERROR_INVALID_MEAS_STATUS);
    }
    
    CurrentTime = clock() / CLOCKS_PER_SEC;
    
    if ((CurrentTime - StartTime) > POWMPR_TIMEOUT)
    {
      // Abort the sensitivity and the combined power/modulation measurement:
      CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
      CheckCmuError (RSCMU_GSM_Sig_AborPowMpr (g_hRsCmuGsm900_Sig, errorMessage));
      CheckCmuError (RSCMU_ERROR_TIMEOUT);
    }
  } while (l_vPowMprStatus != RSCMU_MEAS_RDY);
  
  // Get the combined power/modulation results:
  CheckCmuError (RSCMU_GSM_Sig_FetcPowMpr (g_hRsCmuGsm900_Sig,
                                           RSCMU_MAXMIN,
                                           &l_vAvgBurstPwCurr,
                                           &l_vPeakBurstPwCurr,
                                           &l_vPowerControlLevel,
                                           &l_vTimingAdvError,
                                           &l_vBurstsOutOfPowTol,
                                           &l_vBurstMatching,
                                           &l_vPhErrPeakMaxMin,
                                           &l_vPhErrRMSMaxMin,
                                           &l_vFreqErrMaxMin,
                                           &l_vAvgBurstPwAvg,
                                           errorMessage));
  
  //WriteCmuViReal64Value ("\tAvg. Burst Power (Avg.)[dBm] = ", l_vAvgBurstPwAvg);
  WriteCmuViReal64Value ("\tPeak Burst Power       [dBm] = ", l_vPeakBurstPwCurr);
  WriteCmuViReal64Value ("\tMaximum phase error peak [] = ", l_vPhErrPeakMaxMin);
  WriteCmuViReal64Value ("\tMaximum phase error RMS  [] = ", l_vPhErrRMSMaxMin);
  WriteCmuViReal64Value ("\tMaximum frequecy error  [Hz] = ", l_vFreqErrMaxMin);
  
  // Abort the combined power/modulation measurements:
  CheckCmuError (RSCMU_GSM_Sig_AborPowMpr (g_hRsCmuGsm900_Sig, errorMessage));
  
  StartTime = clock() / CLOCKS_PER_SEC;
  
  // Wait until the sensitivity measurement is in the RDY (ready) state:
  do
  {
  CheckCmuError (RSCMU_GSM_Sig_FetcSensStat (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, &l_vSensStatus, &l_vCurrentStatCycleCount, &l_vCurrentEvalPeriodCount, errorMessage));
  switch (l_vSensStatus)
  {
    case RSCMU_MEAS_RUN:
      break;
    case RSCMU_MEAS_RDY:
      break;
    default:
      // Abort the sensitivity measurement:
      CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
      CheckCmuError (RSCMU_ERROR_INVALID_MEAS_STATUS);
  }
    
    CurrentTime = clock() / CLOCKS_PER_SEC;
    
    if ((CurrentTime - StartTime) > BER_TIMEOUT)
    {
      // Abort the sensitivity measurement:
      CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
      CheckCmuError (RSCMU_ERROR_TIMEOUT);
    }
    
    if (l_vSensMode == RSCMU_RFER)
    {
      // Get Rx Level and Rx Quality reported by the mobile:
      CheckCmuError (RSCMU_GSM_Sig_SensRrepRxl (g_hRsCmuGsm900_Sig, &l_vRxLev, errorMessage));
      CheckCmuError (RSCMU_GSM_Sig_SensRrepRxq (g_hRsCmuGsm900_Sig, &l_vRxQual, errorMessage));
    }
  } while (l_vSensStatus != RSCMU_MEAS_RDY);
  
  if (l_vSensMode == RSCMU_RFER)
  {
    WriteCmuViInt32Value ("\tRX Level (-102.0 dBm)        = ", l_vRxLev);
    WriteCmuViInt32Value ("\tRX Quality                   = ", l_vRxQual);
  }
  
  // Get the results of the bit error rate test.
  switch (l_vSensMode)
  {
    case RSCMU_RFER :
      CheckCmuError (RSCMU_GSM_Sig_Rfer_FetcSens (g_hRsCmuGsm900_Sig,
                                                  RSCMU_SINGLE_SHOT,
                                                  &l_vProgressTime,
                                                  &l_vClassIIBits,
                                                  &l_vClassIbBits,
                                                  &l_vFer,
                                                  &l_vCRCErrors,
                                                  &l_vStatus,
                                                  errorMessage));
      WriteCmuViReal64Value ("\tClass II bits            [%] = ", l_vClassIIBits);
      WriteCmuViReal64Value ("\tFER                      [%] = ", l_vFer);
    break;
    case RSCMU_BBB :
      CheckCmuError (RSCMU_GSM_Sig_Bbb_FetcSens (g_hRsCmuGsm900_Sig,
                                                 RSCMU_SINGLE_SHOT,
                                                 &l_vProgressTime,
                                                 &l_vClassIIBits,
                                                 &l_vCRCErrors,
                                                 &l_vStatus,
                                                 errorMessage));
      WriteCmuViReal64Value ("\tClass II bits            [%] = ", l_vClassIIBits);
    break;
  }
  
  // Abort the sensitivity measurement:
  CheckCmuError (RSCMU_GSM_Sig_AborSens (g_hRsCmuGsm900_Sig, RSCMU_SINGLE_SHOT, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //      Dualband Handover GSM900 to GSM1800, Channel 512, PCL 0
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_Sig_HandOver (g_hRsCmuGsm900_Sig, g_hRsCmuGsm1800_Sig, errorMessage));
  
  printf ("GSM 1800 Band\n");
  printf ("-------------\n");
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //      Channel 512, PCL 0
  //
  // Read out the single shot modulation measurement.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  printf ("\tTCH 512, PCL 0\n");
  printf ("\t--------------\n");
  
  // Read out the modulation measurement results:
  CheckCmuError (RSCMU_GSM_Sig_ReadMod (g_hRsCmuGsm1800_Sig,
                                        RSCMU_MAXMIN,
                                        &l_vPhErrPeakMaxMin,
                                        &l_vPhErrRMSMaxMin,
                                        &l_vFreqErrMaxMin,
                                        &l_vAvgBurstPwCurr,
                                        &l_vBurstsOutOfModTol,
                                        errorMessage));
  
  WriteCmuViReal64Value ("\tMaximum phase error peak [] = ", l_vPhErrPeakMaxMin);
  WriteCmuViReal64Value ("\tMaximum phase error RMS  [] = ", l_vPhErrRMSMaxMin);
  WriteCmuViReal64Value ("\tMaximum frequecy error  [Hz] = ", l_vFreqErrMaxMin);
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //      Channel 885, PCL 10
  //
  // Read out the single shot sensitivity measurement.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  printf ("\tTCH 885, PCL 10\n");
  printf ("\t--------------\n");
  
  // Change the traffic channel and the PCL of the mobile without dropping the
  // call:
  CheckCmuError (RSCMU_GSM_Sig_ProcSignChcc (g_hRsCmuGsm1800_Sig, 885, 3, 10, errorMessage));
  
  // Get the results of the bit error rate test.
  switch (l_vSensMode)
  {
    case RSCMU_RFER :
      CheckCmuError (RSCMU_GSM_Sig_Rfer_ReadSens (g_hRsCmuGsm1800_Sig,
                                                  RSCMU_SINGLE_SHOT,
                                                  &l_vProgressTime,
                                                  &l_vClassIIBits,
                                                  &l_vClassIbBits,
                                                  &l_vFer,
                                                  &l_vCRCErrors,
                                                  &l_vStatus,
                                                  errorMessage));
      WriteCmuViReal64Value ("\tClass II bits            [%] = ", l_vClassIIBits);
      WriteCmuViReal64Value ("\tFER                      [%] = ", l_vFer);
    break;
    case RSCMU_BBB :
      CheckCmuError (RSCMU_GSM_Sig_Bbb_ReadSens (g_hRsCmuGsm1800_Sig,
                                                 RSCMU_SINGLE_SHOT,
                                                 &l_vProgressTime,
                                                 &l_vClassIIBits,
                                                 &l_vCRCErrors,
                                                 &l_vStatus,
                                                 errorMessage));
      WriteCmuViReal64Value ("\tClass II bits            [%] = ", l_vClassIIBits); 
    break;
  }
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //      Call release in GSM1800
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_GSM_Sig_ProcSignAct (g_hRsCmuGsm1800_Sig, RSCMU_CREL, errorMessage));
  
  do {
    CheckCmuError (RSCMU_GSM_Sig_SensSignStat (g_hRsCmuGsm1800_Sig, &l_vSignallingState, errorMessage));
  } while (l_vSignallingState != RSCMU_STATE_SOFF);
  
  CheckCmuError (RSCMU_GSM_Sig_SensSignStat (g_hRsCmuGsm1800_Sig, &l_vSignallingState, errorMessage));
  CheckCmuError (RSCMU_GSM_Sig_SensSignStat (g_hRsCmuGsm900_Sig, &l_vSignallingState, errorMessage));

  printf ("\n");
  printf ("... Test successfully passed\n");
  
  // Close the file "RsCmu.iee".
  CheckCmuError (RSCMU_Trace_OFF (g_hRsCmuBase, errorMessage));
  
  CheckCmuError (RSCMU_Close_RF_NSig (g_hRsCmuRf_NSig, errorMessage));
  CheckCmuError (RSCMU_Close_GSM_Sig (g_hRsCmuGsm900_Sig, errorMessage));
  CheckCmuError (RSCMU_Close_GSM_NSig (g_hRsCmuGsm900_NSig, errorMessage));
  CheckCmuError (RSCMU_Close_GSM_Sig (g_hRsCmuGsm1800_Sig, errorMessage));
  CheckCmuError (RSCMU_Close_GSM_NSig (g_hRsCmuGsm1800_NSig, errorMessage));
  CheckCmuError (RSCMU_Close_Base (g_hRsCmuBase, errorMessage));

  return (0);

Error:
  return (-1);
}


